@@ -124,6 +124,31 @@ static void add_slash(char *dest, const char *src, int len_max, int len)
 	return;
 }
 
+static void get_escaped_fields(char **fields, ...)
+{
+	va_list ap;
+	GString *line;
+	char *field;
+	char escaped[LEN_MAX];
+
+	va_start(ap, fields);
+	line = g_string_new("");
+
+	for (field = va_arg(ap, char *); field; ) {
+		add_slash(escaped, field, LEN_MAX, strlen(field));
+		g_string_append(line, escaped);
+
+		field = va_arg(ap, char *);
+
+		if (field)
+			g_string_append(line, ";");
+	}
+
+	va_end(ap);
+
+	*fields = g_string_free(line, FALSE);
+}
+
 static void vcard_printf_begin(GString *vcards, uint8_t format)
 {
 	vcard_printf(vcards, "BEGIN:VCARD");
@@ -177,6 +202,8 @@ gboolean address_fields_present(const char *address)
 static void vcard_printf_name(GString *vcards,
 					struct phonebook_contact *contact)
 {
+	char *fields;
+
 	if (contact_fields_present(contact) == FALSE) {
 		/* If fields are empty, add only 'N:' as parameter.
 		 * This is crucial for some devices (Nokia BH-903) which
@@ -189,9 +216,15 @@ static void vcard_printf_name(GString *vcards,
 		return;
 	}
 
-	vcard_printf(vcards, "N:%s;%s;%s;%s;%s", contact->family,
+
+	get_escaped_fields(&fields, contact->family,
 				contact->given, contact->additional,
-				contact->prefix, contact->suffix);
+				contact->prefix, contact->suffix,
+				NULL);
+
+	vcard_printf(vcards, "N:%s", fields);
+
+	g_free(fields);
 }
 
 static void vcard_printf_fullname(GString *vcards, const char *text)
@@ -256,34 +289,7 @@ static void vcard_printf_number(GString *vcards, uint8_t format,
 	vcard_printf(vcards, buf, number);
 }
 
-static void vcard_printf_tag(GString *vcards, uint8_t format, const char *tag,
-					const char *category, const char *fld)
-{
-	char *separator = "", *type = "";
-	char buf[LEN_MAX];
-
-	if (tag == NULL || strlen(tag) == 0)
-		return;
-
-	if (fld == NULL || strlen(fld) == 0){
-		vcard_printf(vcards, "%s:", tag);
-		return;
-	}
-
-	if (category && strlen(category)) {
-		separator = ";";
-		if (format == FORMAT_VCARD30)
-			type = "TYPE=";
-	} else {
-		category = "";
-	}
-
-	snprintf(buf, LEN_MAX, "%s%s%s%s", tag, separator, type, category);
-
-	vcard_printf(vcards, "%s:%s", buf, fld);
-}
-
-static void vcard_printf_slash_tag(GString *vcards, uint8_t format,
+static void vcard_printf_tag(GString *vcards, uint8_t format,
 					const char *tag, const char *category,
 					const char *fld)
 {
@@ -294,7 +300,7 @@ static void vcard_printf_slash_tag(GString *vcards, uint8_t format,
 	if (tag == NULL || strlen(tag) == 0)
 		return;
 
-	if (fld == NULL || (len = strlen(fld)) == 0){
+	if (fld == NULL || (len = strlen(fld)) == 0) {
 		vcard_printf(vcards, "%s:", tag);
 		return;
 	}
@@ -321,11 +327,11 @@ static void vcard_printf_email(GString *vcards, uint8_t format,
 	char field[LEN_MAX];
 	int len = 0;
 
-	if (!address || !(len = strlen(address))){
+	if (!address || !(len = strlen(address))) {
 		vcard_printf(vcards, "EMAIL:");
 		return;
 	}
-	switch (category){
+	switch (category) {
 	case FIELD_TYPE_HOME:
 		if (format == FORMAT_VCARD21)
 			category_string = "INTERNET;HOME";
@@ -346,7 +352,7 @@ static void vcard_printf_email(GString *vcards, uint8_t format,
 	}
 
 	add_slash(field, address, LEN_MAX, len);
-	vcard_printf(vcards,"EMAIL;%s:%s", category_string, field);
+	vcard_printf(vcards, "EMAIL;%s:%s", category_string, field);
 }
 
 static void vcard_printf_url(GString *vcards, uint8_t format,
@@ -354,6 +360,7 @@ static void vcard_printf_url(GString *vcards, uint8_t format,
 					enum phonebook_field_type category)
 {
 	const char *category_string = "";
+	char field[LEN_MAX];
 
 	if (!url || strlen(url) == 0) {
 		vcard_printf(vcards, "URL:");
@@ -381,7 +388,8 @@ static void vcard_printf_url(GString *vcards, uint8_t format,
 		break;
 	}
 
-	vcard_printf(vcards,"URL;%s:%s", category_string, url);
+	add_slash(field, url, LEN_MAX, strlen(url));
+	vcard_printf(vcards, "URL;%s:%s", category_string, field);
 }
 
 static gboolean org_fields_present(struct phonebook_contact *contact)
@@ -398,13 +406,17 @@ static gboolean org_fields_present(struct phonebook_contact *contact)
 static void vcard_printf_org(GString *vcards,
 					struct phonebook_contact *contact)
 {
-	if (org_fields_present(contact) == FALSE){
-		vcard_printf(vcards, "ORG:");
-		return;
-	}
+	char *fields;
 
-	vcard_printf(vcards, "ORG:%s;%s", contact->company,
-				contact->department);
+	if (org_fields_present(contact) == FALSE)
+		return;
+
+	get_escaped_fields(&fields, contact->company,
+					contact->department, NULL);
+
+	vcard_printf(vcards, "ORG:%s", fields);
+
+	g_free(fields);
 }
 
 static void vcard_printf_address(GString *vcards, uint8_t format,
@@ -505,13 +517,14 @@ void phonebook_add_contact(GString *vcards, struct phonebook_contact *contact,
 
 	vcard_printf_begin(vcards, format);
 
-	if (filter & FILTER_UID)
+	if (filter & FILTER_UID && *contact->uid)
 		vcard_printf_tag(vcards, format, "UID", NULL, contact->uid);
 
 	if (filter & FILTER_N)
 		vcard_printf_name(vcards, contact);
 
-	if (filter & FILTER_FN)
+	if (filter & FILTER_FN && (*contact->fullname ||
+					format == FORMAT_VCARD30))
 		vcard_printf_fullname(vcards, contact->fullname);
 
 	if (filter & FILTER_TEL) {
@@ -532,11 +545,7 @@ void phonebook_add_contact(GString *vcards, struct phonebook_contact *contact,
 	if (filter & FILTER_EMAIL) {
 		GSList *l = contact->emails;
 
-		if (g_slist_length(l) == 0)
-			vcard_printf_email(vcards, format, NULL,
-							FIELD_TYPE_OTHER);
-
-		for (; l; l = l->next){
+		for (; l; l = l->next) {
 			struct phonebook_field *email = l->data;
 			vcard_printf_email(vcards, format, email->text,
 								email->type);
@@ -546,10 +555,6 @@ void phonebook_add_contact(GString *vcards, struct phonebook_contact *contact,
 	if (filter & FILTER_ADR) {
 		GSList *l = contact->addresses;
 
-		if (g_slist_length(l) == 0)
-			vcard_printf_address(vcards, format, NULL,
-							FIELD_TYPE_OTHER);
-
 		for (; l; l = l->next) {
 			struct phonebook_field *addr = l->data;
 			vcard_printf_address(vcards, format, addr->text,
@@ -557,38 +562,34 @@ void phonebook_add_contact(GString *vcards, struct phonebook_contact *contact,
 		}
 	}
 
-	if (filter & FILTER_BDAY)
+	if (filter & FILTER_BDAY && *contact->birthday)
 		vcard_printf_tag(vcards, format, "BDAY", NULL,
 						contact->birthday);
 
-	if (filter & FILTER_NICKNAME)
-		vcard_printf_slash_tag(vcards, format, "NICKNAME", NULL,
+	if (filter & FILTER_NICKNAME && *contact->nickname)
+		vcard_printf_tag(vcards, format, "NICKNAME", NULL,
 							contact->nickname);
 
 	if (filter & FILTER_URL) {
 		GSList *l = contact->urls;
 
-		if (g_slist_length(l) == 0)
-			vcard_printf_url(vcards, format, NULL,
-							FIELD_TYPE_OTHER);
-
 		for (; l; l = l->next) {
 			struct phonebook_field *url = l->data;
 			vcard_printf_url(vcards, format, url->text, url->type);
 		}
 	}
 
-	if (filter & FILTER_PHOTO)
+	if (filter & FILTER_PHOTO && *contact->photo)
 		vcard_printf_tag(vcards, format, "PHOTO", NULL,
 							contact->photo);
 
 	if (filter & FILTER_ORG)
 		vcard_printf_org(vcards, contact);
 
-	if (filter & FILTER_ROLE)
+	if (filter & FILTER_ROLE && *contact->role)
 		vcard_printf_tag(vcards, format, "ROLE", NULL, contact->role);
 
-	if (filter & FILTER_TITLE)
+	if (filter & FILTER_TITLE && *contact->title)
 		vcard_printf_tag(vcards, format, "TITLE", NULL, contact->title);
 
 	if (filter & FILTER_X_IRMC_CALL_DATETIME)
